/* ---------------------------------------------------------------------------------------------- *
 * adc.c                                                                                          *
 *                                                                                                *
 * Created: 5/06/2014 10:45:42 AM                                                                 *
 *  Author: David                                                                                 *
 * ---------------------------------------------------------------------------------------------- */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdbool.h>
#include "main.h"

/* ---------------------------------------------------------------------------------------------- *
 * ADC inputs: oven & controller temperatures                                                     *
 * Reference voltage = 2048mV, so 10-bit ADC is 2mV/step                                          *
 * ADC clock = Fosc/128 = 7.8kHz  Tconv = 125msec                                                 *
 * ---------------------------------------------------------------------------------------------- */
// Kick off an ADC conversion: interrupt when done
void AdcStartConversion(uint8_t chan)
{
	ADMUX   = chan;
	ADCSRA |= BV(ADSC);
}

// Start the ADC, reading (only) channels 0 & 1, in the background
void AdcStartup(void)
{		//   ADC on     ADC int.   Prescaler = 128
	ADCSRA = BV(ADEN) | BV(ADIE) | Config.AdcPrescale_u8;
	AdcStartConversion(0);
}

// ADC interrupt function
SIGNAL(ADC_vect)
{
	switch (ADMUX)					// Channel just read
	{
		// Controller PCB temperature: LM45CIM3, 10mV ie 5 steps / degC
	case 0:
		ControllerTemp_u16 = ADC / 5;
		AdcStartConversion(1);		
		break;
		
		// Oven temperature: AD8495 + K-type thermocouple, 5mV ie 2.5 steps / degC
	case 1:
		OvenTemp_u16 = (ADC * 2) / 5;
		AdcStartConversion(0);
		break;
		
		// Fault: restart sequence
	default:
		AdcStartup();
		break;
	}
}
