/* ---------------------------------------------------------------------------------------------- *
 * buttons.c                                                                                      *
 *                                                                                                *
 * Created: 6/06/2014 12:15:44 PM                                                                 *
 *  Author: David                                                                                 *
 * ---------------------------------------------------------------------------------------------- */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdbool.h>
#include "main.h"

static bool Button_ab[NUMBUTTONS] = { false };	// Filtered button states
static bool Edge_ab[NUMBUTTONS]	  = { false };	// Button has changed

/* ---------------------------------------------------------------------------------------------- *
 * Periodic update of button status                                                               *
 * ---------------------------------------------------------------------------------------------- */
void ButtonUpdate(void)
{
	static bool	   old_b[NUMBUTTONS] = { false, false, false, false };
		   bool    new_b[NUMBUTTONS];
		   uint8_t i;
	
	new_b[0] = !(PINB & BT0_);		// Active-low inputs
	new_b[1] = !(PIND & BT1_);
	new_b[2] = !(PINB & BT2_);
	new_b[3] = !(PIND & BT3_);
	
	for(i=0; i<NUMBUTTONS; i++)
	{
		if (new_b[i] == old_b[i])
		{							// Not a bounce: stable value
			if (new_b[i] != Button_ab[i])
			{						// A valid change
				Button_ab[i] = new_b[i];
				Edge_ab[i]   = true;
			} 
			else
			{						// Nothing to see here...
				Edge_ab[i]   = false;
			}
		} 
		else
		{							// Change: glitch filter
			old_b[i] = new_b[i];
		}
	}
}

/* ---------------------------------------------------------------------------------------------- *
 * Once-only value: button pressed                                                                *
 * ---------------------------------------------------------------------------------------------- */
bool ButtonPressed(uint8_t btn_u8)
{
	if (btn_u8 < NUMBUTTONS)
	{
		return (Edge_ab[btn_u8] & Button_ab[btn_u8]);
	} 
	else
	{
		return false;
	}
}

/* ---------------------------------------------------------------------------------------------- *
 * Once-only value: button released                                                               *
 * ---------------------------------------------------------------------------------------------- */
bool ButtonReleased(uint8_t btn_u8)
{
	if (btn_u8 < NUMBUTTONS)
	{
		return (Edge_ab[btn_u8] & !Button_ab[btn_u8]);
	}
	else
	{
		return false;
	}
}

/* ---------------------------------------------------------------------------------------------- *
 * Get current button state                                                                       *
 * ---------------------------------------------------------------------------------------------- */
bool ButtonHeld(uint8_t btn_u8)
{
	if (btn_u8 < NUMBUTTONS)
	{
		return Button_ab[btn_u8];
	}
	else
	{
		return false;
	}
}
