/* ---------------------------------------------------------------------------------------------- *
 * display.c                                                                                      *
 *                                                                                                *
 * Created: 5/06/2014 10:54:44 AM                                                                 *
 *  Author: David                                                                                 *
 * ---------------------------------------------------------------------------------------------- */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdbool.h>
#include "main.h"

/* ---------------------------------------------------------------------------------------------- *
 * 7-Segment LED display: 3 digits, left: 0 - 1 - 2 :right                                        *
 * ---------------------------------------------------------------------------------------------- */
bool   BlinkGate_b = true;		// Set by timer interrupt

static uint8_t  Display_au8[3] = { '0',   '0',   '0'   };		// 7-segt. display (encoded)
static bool     Blink_ab[3]	   = { false, false, false };

#define SEG_DP (0x01)	// Sequence to facilitate PCB routing
#define SEG_A  (0x10)
#define SEG_B  (0x08)
#define SEG_C  (0x20)
#define SEG_D  (0x04)
#define SEG_E  (0x40)
#define SEG_F  (0x02)
#define SEG_G  (0x80)

#define BLINK7 0x80
#define FAULT7 0xFF			// Bad char.: all segts. blink

static const uint8_t segtab[128] =
{
	FAULT7												 ,	// 00
	FAULT7												 ,	// 01
	FAULT7												 ,	// 02
	FAULT7												 ,	// 03
	FAULT7												 ,	// 04
	FAULT7												 ,	// 05
	FAULT7												 ,	// 06
	FAULT7												 ,	// 07
	FAULT7												 ,	// 08
	FAULT7												 ,	// 09
	FAULT7												 ,	// 0A
	FAULT7												 ,	// 0B
	FAULT7												 ,	// 0C
	FAULT7												 ,	// 0D
	FAULT7												 ,	// 0E
	FAULT7												 ,	// 0F
	FAULT7												 ,	// 10
	FAULT7												 ,	// 11
	FAULT7												 ,	// 12
	FAULT7												 ,	// 13
	FAULT7												 ,	// 14
	FAULT7												 ,	// 15
	FAULT7												 ,	// 16
	FAULT7												 ,	// 17
	FAULT7												 ,	// 18
	FAULT7												 ,	// 19
	FAULT7												 ,	// 1A
	FAULT7												 ,	// 1B
	FAULT7												 ,	// 1C
	FAULT7												 ,	// 1D
	FAULT7												 ,	// 1E
	FAULT7												 ,	// 1F
	0													 ,	// 20
	FAULT7												 ,	// 21
	FAULT7												 ,	// 22
	FAULT7												 ,	// 23
	FAULT7												 ,	// 24
	FAULT7												 ,	// 25
	FAULT7												 ,	// 26
	FAULT7												 ,	// 27
	FAULT7												 ,	// 28
	FAULT7												 ,	// 29
	FAULT7												 ,	// 2A
	FAULT7												 ,	// 2B
	FAULT7												 ,	// 2C
	                                                SEG_G,	// 2D
	FAULT7												 ,	// 2E
	FAULT7												 ,	// 2F
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F        ,	// 30
	SEG_B |			SEG_C								 ,	// 31
	SEG_A | SEG_B |         SEG_D | SEG_E |         SEG_G,	// 32
	SEG_A | SEG_B | SEG_C | SEG_D |                 SEG_G,	// 33
	        SEG_B | SEG_C |                 SEG_F | SEG_G,	// 34
	SEG_A |         SEG_C | SEG_D |         SEG_F | SEG_G,	// 35
	SEG_A |         SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,	// 36
	SEG_A | SEG_B | SEG_C								 ,	// 37
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,	// 38
	SEG_A | SEG_B | SEG_C | SEG_D |         SEG_F | SEG_G,	// 39
	FAULT7												 ,	// 3A
	FAULT7												 ,	// 3B
	FAULT7												 ,	// 3C
	FAULT7												 ,	// 3D
	FAULT7												 ,	// 3E
	FAULT7												 ,	// 3F
	FAULT7												 ,	// 40
	SEG_A | SEG_B | SEG_C |         SEG_E | SEG_F | SEG_G,	// 41
	                SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,	// 42
	SEG_A |                 SEG_D | SEG_E | SEG_F		 ,	// 43
	        SEG_B | SEG_C | SEG_D | SEG_E |         SEG_G,	// 44
	SEG_A |                 SEG_D | SEG_E | SEG_F | SEG_G,	// 45
	SEG_A |                         SEG_E | SEG_F | SEG_G,	// 46
	FAULT7												 ,	// 47
	        SEG_B | SEG_C |         SEG_E | SEG_F | SEG_G,	// 48
	                                SEG_E | SEG_F		 ,	// 49
	        SEG_B | SEG_C | SEG_D                        ,	// 4A
	FAULT7												 ,	// 4B
	                        SEG_D | SEG_E | SEG_F        ,	// 4C
	FAULT7												 ,	// 4D
	FAULT7												 ,	// 4E
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F        ,	// 4F
	SEG_A | SEG_B |                 SEG_E | SEG_F | SEG_G,	// 50
	FAULT7												 ,	// 51
	FAULT7												 ,	// 52
	SEG_A | SEG_B |         SEG_D | SEG_E |         SEG_G,	// 53
	FAULT7												 ,	// 54
	        SEG_B | SEG_C | SEG_D | SEG_E | SEG_F        ,	// 55
	FAULT7												 ,	// 56
	FAULT7												 ,	// 57
	FAULT7												 ,	// 58
	        SEG_B | SEG_C |                 SEG_F | SEG_G,	// 59
	FAULT7												 ,	// 5A
	FAULT7												 ,	// 5B
	FAULT7												 ,	// 5C
	FAULT7												 ,	// 5D
	FAULT7												 ,	// 5E
	FAULT7												 ,	// 5F
	FAULT7												 ,	// 60
	FAULT7												 ,	// 61
	                SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,	// 62
	                        SEG_D | SEG_E |         SEG_G,	// 63
	        SEG_B | SEG_C | SEG_D | SEG_E |         SEG_G,	// 64
	FAULT7												 ,	// 65
	FAULT7												 ,	// 66
	FAULT7												 ,	// 67
	                SEG_C |         SEG_E | SEG_F | SEG_G,	// 68
	FAULT7												 ,	// 69
	        SEG_B | SEG_C | SEG_D                        ,	// 6A
	FAULT7												 ,	// 6B
	                                SEG_E | SEG_F        ,	// 6C
	FAULT7												 ,	// 6D
	                SEG_C |         SEG_E |         SEG_G,	// 6E
	                SEG_C | SEG_D | SEG_E |         SEG_G,	// 6F
	SEG_A | SEG_B |                 SEG_E | SEG_F | SEG_G,	// 70
	FAULT7												 ,	// 71
	                                SEG_E |         SEG_G,	// 72
	FAULT7												 ,	// 73
	                        SEG_D | SEG_E | SEG_F | SEG_G,	// 74
	                SEG_C | SEG_D | SEG_E                ,	// 75
	FAULT7												 ,	// 76
	FAULT7												 ,	// 77
	FAULT7												 ,	// 78
	FAULT7												 ,	// 79
	FAULT7												 ,	// 7A
	FAULT7												 ,	// 7B
	FAULT7												 ,	// 7C
	FAULT7												 ,	// 7D
	FAULT7												 ,	// 7E
	FAULT7												  	// 7F
};

/* ---------------------------------------------------------------------------------------------- *
 * ASCII char. -> 7-segment display, in given position.                                           *
 * Blink & decimal-pt. are optional                                                               *
 * ---------------------------------------------------------------------------------------------- */
void putchr(uint8_t chr_u8, bool dp_b, bool blink_b, uint8_t pos_u8)
{
	if(pos_u8 < sizeof(Display_au8))			// Better be a valid position!
	{
		uint8_t segmap;
		
		segmap = segtab[chr_u8 & 0x7f];
		if (dp_b)
		{
			segmap |= SEG_DP;
		}
		Display_au8[pos_u8] = segmap;
		Blink_ab[pos_u8]    = blink_b;
	}
}

/* ---------------------------------------------------------------------------------------------- *
 * ASCIIZ string (max. 3 chars.) displayed left-justified.                                        *
 * Blink optional.                                                                                *
 * ---------------------------------------------------------------------------------------------- */
void putstr(const char *msg_pu8, bool blink_b)
{
	int8_t i;
	const char blank_au8[] = "   ";
	
	for(i=0; i<3; i++)
	{
		if (*msg_pu8 == '\0')
		{
			msg_pu8 = blank_au8;
		}
		
		putchr(*(msg_pu8++), false, blink_b, i);
	}
}

/* ---------------------------------------------------------------------------------------------- *
 * Set/clear the decimal point                                                                    *
 * ---------------------------------------------------------------------------------------------- */
void setdecimal(uint8_t pos_u8, bool dp_b)
{
	if (pos_u8 < 3)
	{
		if (dp_b)
		{
			Display_au8[pos_u8] |= SEG_DP;
		} 
		else
		{
			Display_au8[pos_u8] &= ~SEG_DP;
		}
	}
}
	
/* ---------------------------------------------------------------------------------------------- *
 * Display unsigned number, right justified.                                                      *
 * ---------------------------------------------------------------------------------------------- */
void putnum(uint16_t val_u16)
{
	if (val_u16 > 999)
	{
		putstr("HI", false);
	} 
	else
	if (val_u16 == 0)
	{
		putstr("  0", false);
	}
	else
	{		// We know it's 0..999
		int8_t i;
		for(i=2; (val_u16 > 0) && (i >= 0); val_u16 /= 10)
		{
			putchr('0'+(val_u16 % 10), false, false, i--);
		}
		while(i >= 0)
		{
			putchr(' ', false, false, i--);
		}
	}
}

/* ---------------------------------------------------------------------------------------------- *
 * Get the raw value to display (used by the Timer)                                               *
 * ---------------------------------------------------------------------------------------------- */
uint8_t getchr(uint8_t pos_u8)
{
	if (pos_u8 < sizeof(Display_au8))
	{
		if(Blink_ab[pos_u8] && BlinkGate_b)
		{
			return 0;
		}
		else
		{
			return Display_au8[pos_u8];
		}			
	}
	else
	{
		return 0;
	}
}
