/* ---------------------------------------------------------------------------------------------- *
 * main.h                                                                                         *
 *                                                                                                *
 * Created: 5/06/2014 10:43:49 AM                                                                 *
 *  Author: David                                                                                 *
 * ---------------------------------------------------------------------------------------------- */

#ifndef MAIN_H_
#define MAIN_H_

extern void		AdcStartConversion(uint8_t chan);
extern void		AdcStartup(void);

extern void		putchr(uint8_t chr_u8, bool dp_b, bool blink_b, uint8_t pos_u8);
extern void		putstr(const char *msg_pu8, bool blink_b);
extern void		putnum(uint16_t val_u16);
extern uint8_t	getchr(uint8_t  pos_u8);
extern void     setdecimal(uint8_t pos_u8, bool dp_b);

extern void		SpiStartup(void);
extern void		SpiSetPin(uint8_t bit_u8, bool val_b);
extern bool		SpiGetPin(uint8_t bit_u8);

extern void		ButtonUpdate(void);
extern bool		ButtonPressed(uint8_t btn_u8);
extern bool		ButtonReleased(uint8_t btn_u8);
extern bool		ButtonHeld(uint8_t btn_u8);

extern uint16_t OvenTemp_u16;
extern uint16_t ControllerTemp_u16;
extern bool     BlinkGate_b;

#define NUMBUTTONS 4			// No. of LEDs/buttons on panel

#define BV(x) (1 << (x))

// Port-B bit assignments
#define BT0_		BV(0)		// I Button 0 input
#define EN_			BV(1)		// O Main output enable
#define LD			BV(2)		// O Load '595 output registers
#define MOSI		BV(3)		// O '595 serial data
#define MISO		BV(4)		// - Debug only
#define SCK			BV(5)		// O '595 shift clock
#define LD2_		BV(6)		// O Button 2 LED
#define BT2_		BV(7)		// I Button 2 input

// Port-C bit assignments
#define BRD_TEMP	BV(0)		// I Controller PCB temperature
#define OVEN_TEMP	BV(1)		// I Oven temperature

// Port-D bit assignments
#define RXD			BV(0)		// I RS485 input
#define TXD			BV(1)		// O RS485 output
#define DE			BV(2)		// O RS485 transmit
#define LD3_		BV(3)		// O Button 3 LED
#define BT3_		BV(4)		// I Button 3 input
#define LD1_		BV(5)		// O Button 1 LED
#define BT1_		BV(6)		// I Button 1 input
#define LD0_		BV(7)		// O Button 0 LED

enum					// Bit indices in the SPI word
{
						// Bit-0 not used
	SPI_HTR1	= 1,	// Oven heater 1 (top)
	SPI_ACFAN,			// Oven circulating fan (hot air)
	SPI_HTR2,			// Oven heater 2 (bottom)
	SPI_DCFAN,			// Control-box fan (cold air)
	SPI_DIGIT2,			// Display digits
	SPI_DIGIT1,
	SPI_DIGIT0
};

#define DGT0   (1 << SPI_DIGIT0)
#define DGT1   (1 << SPI_DIGIT1)
#define DGT2   (1 << SPI_DIGIT2)

typedef struct			// Setpoint on temperature profile 
{
	uint16_t	secs_u16;		// Elapsed time
	uint16_t	celsius_u16;	// Temperature
} S_SPOINT;

typedef struct			// Main configuration (load from EEPROM?)
{
	S_SPOINT	Lead_s[6];			// Profile: leaded
	S_SPOINT	UnLead_s[6];		// Profile: unleaded
	uint16_t	WarmController_u16; // Start the controller fan
	uint16_t	CoolController_u16;	// Stop the fan
	uint16_t	HotController_u16;	// Too hot: must STOP
	uint16_t	OvenFanOnTemp_u16;	// AC fan runs
	uint16_t	OvenFanOffTemp_u16;	// AC fan stops
	uint16_t	TempOvershoot1_u16;	// Overshoot after cutting power: 100C
	uint16_t	TempOvershoot2_u16;	// Overshoot after cutting power: 200C
	uint16_t    T1Adivide_u16;		// Timer 1A divisor
	uint8_t     T1Aprescale_u8;		// Timer 1A prescaler
	uint8_t		AdcPrescale_u8;		// ADC prescaler bits
	uint8_t		SpiPrescale_u8;		// SPI prescaler bits
} S_CONFIG;

extern S_CONFIG Config;			// Master config. record

#endif /* MAIN_H_ */