/* ---------------------------------------------------------------------------------------------- *
 *                                                                                                *
 *   Module      :   PID controller code                                                          *
 *   Copyright   :   (C)2004 Kermit Enterprises                                                   *
 *   Platform(s) :   Generic                                                                      *
 *   Limitations :                                                                                *
 *   $Workfile: pid.h $                                                                           *
 *   $Revision: 1.0 $                                                                             *
 *                                                                                                *
 *----------------------------------------------------------------------------------------------- *
 *                                                                                                *
 *   Purpose     :   The same old story!                                                          *
 *                                                                                                *
 * ---------------------------------------------------------------------------------------------- */

// 2007.03.04 20:25  [YYYY.MM.DD] Last edited (CodeWright)
// 2014.06.05 10:06  DRB changed to generic format

#ifndef PID_H_
#define PID_H_

/*----- DEFINES --------------------------------------------------------------------------------- */

#define PID_GAIN_SCALING   (16)
#define PID_MAX_OUTPUT   (1023)     // Highest meaningful output
#define PID_D_FILTER        (8)     // (N-1)/N filter on D-term

/*----- TYPEDEFS -------------------------------------------------------------------------------- */

typedef struct      // PID Calibration constants
{   // Input values
    int16_t       feedback_value_s16;  // Input value (for info. only)
    int16_t       setpoint_s16;        // setpoint        
    // Gains (scaled by PID_GAIN_SCALING)
    int16_t       calibration_p_gain_s16;
    int16_t       calibration_i_gain_s16;
    int16_t       calibration_d_gain_s16;
    // Bases for calculated limits
    int16_t       calibration_i_max_s16;
    int16_t       calibration_d_max_s16;
    // End of constants
    // Values computed at start-up
    int32_t       calc_d_max_s32;
    int32_t       calc_i_max_s32;
    // Realtime part, zero-ed at start-up
    int16_t       error_s16;
    int32_t       d_sum_s32;
    int16_t       result_s16;
    // Starting integral, saved from last run
    int32_t       i_sum_s32;               
} S_PID;

/*----- PROTOTYPES ------------------------------------------------------------------------------ */
extern void    PidStartup(S_PID *pid_ptr);
extern int16_t PidCalculate(S_PID *pid_ptr, int16_t fbval_s16);

static inline void PidSetpoint(S_PID *pid_ptr, int16_t sp_s16){pid_ptr->setpoint_s16 = sp_s16;}

#endif /* PID_H_ */
