/* ---------------------------------------------------------------------------------------------- *
 * spi.c                                                                                          *
 *                                                                                                *
 * Created: 5/06/2014 11:06:18 AM                                                                 *
 *  Author: David                                                                                 *
 * ---------------------------------------------------------------------------------------------- */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdbool.h>
#include "main.h"

uint8_t  SpiBuffer_u8 = 0;		// Buffer data, written in background

/* ---------------------------------------------------------------------------------------------- *
 * Set/clear a given output bit, via SPI                                                          *
 * ---------------------------------------------------------------------------------------------- */
void SpiSetPin(uint8_t bit_u8, bool val_b)
{
    uint8_t set = BV(bit_u8);
    if (val_b)
    {
        SpiBuffer_u8 |= set;
    }
    else
    {
        SpiBuffer_u8 &= ~set;
    }
}

/* ---------------------------------------------------------------------------------------------- *
 * Read state of a SPI-driven pin                                                                 *
 * ---------------------------------------------------------------------------------------------- */
bool SpiGetPin(uint8_t bit_u8)
{
    uint8_t val_u8 = SpiBuffer_u8 & BV(bit_u8);
    return (val_u8 != 0);
}
    
/* ---------------------------------------------------------------------------------------------- *
 * SPI driver: power controls & display                                                           *
 * Sends MSB first, bit rate = 7.8kHz (word time = 2048usec)                                      *
 * ---------------------------------------------------------------------------------------------- */
// Start the SPI, data is updated transparently
void SpiStartup(void)
{
    SPCR    = BV(SPIE) | BV(SPE) | BV(MSTR) | Config.SpiPrescale_u8; 
    SPDR    = SpiBuffer_u8;				// This starts a transmission
}

// SPI interrupt: 1 byte sent
SIGNAL(SPI_STC_vect)
{
    static uint8_t SpiByte_u8 = 0;		// State variable
    uint8_t odat_u8;

    odat_u8 = SpiBuffer_u8 & ~(DGT0 | DGT1 | DGT2);	// Drop the digit bits

    switch(SpiByte_u8)
    {
    case 0:								// High half of digit-0		
        SPDR = odat_u8 | DGT0;
        break;
        
    case 1:								// Low half of digit-0
        SPDR = getchr(0);
        break;
        
    case 2:								// High half of digit-1
        SPDR = odat_u8 | DGT1;
        break;
    
    case 3:								// Low half of digit-1
        SPDR = getchr(1);
        break;
    
    case 4:								// High half of digit-2
        SPDR = odat_u8 | DGT2;
        break;
    
    case 5:								// Low half of digit-2 & restart cycle
        SPDR = getchr(2);
        SpiByte_u8 = 0xff;				// Start the next cycle
        break;
        
    default:							// Fault: restart transmission
        SpiByte_u8 = 0xff;
        SpiStartup();
        break;	
    }
    
    if((SpiByte_u8 & 1) == 0)			// Sent 16 bits: transfer to OP latches
    {
        PORTB  |=  LD;					// Output latch
        PORTB  &= ~LD;					// Latch enable LOW
        PORTB  &= ~EN_;					// Enable the output drivers
    }
    SpiByte_u8++;
}
